from .thetao_yz_annual_bias_1x1deg import parse_and_run, parse, run

__description__ = "potential tempertaure depth-latitude annual bias plot"
__ppstreams__ = ["ocean_annual_z_d2_1x1deg/av", "ocean_annual_z_1x1deg/av"]
