from .stratification import parse, read, calculate, plot, run, parse_and_run

__description__ = "Evaluate stratification parameters in z-coordinates"
__ppstreams__ = [
    "ocean_annual_z_1x1deg/av",
]
__ppvars__ = ["thetao"]
