from .template import parse, read, calculate, plot, run, parse_and_run

__description__ = "Template for creatig a new diagnostic"
__ppstreams__ = [
    "ocean_annual_z/av",
]
__ppvars__ = ["some_variable"]
