# -*- coding: utf-8 -*-
"""m6plot.coords - routines to handle array operations associated
with 2-D grids
"""

__all__ = [
    "boundaryStats",
    "expand",
    "expandI",
    "expandJ",
    "section2quadmesh",
]

for func in __all__:
    exec(f"from .{func} import *")
